# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['decision_footnote']

package_data = \
{'': ['*']}

install_requires = \
['lawsql-utils>=0.0.1,<0.0.2']

setup_kwargs = {
    'name': 'decision-footnote',
    'version': '0.0.1',
    'description': 'Split html style sup-footnotes to validated list of dicts',
    'long_description': 'None',
    'author': 'Marcelino G. Veloso III',
    'author_email': 'mars@veloso.one',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



import re
from typing import Iterator, Optional

from bs4.element import ResultSet, Tag
from lawsql_utils.html import (
    get_asterisk_from_fn_pattern,
    get_digit_from_fn_pattern,
    make_soup,
)


def create_notes(annex: str) -> Optional[list[dict]]:
    """Split annex into list of identifiable footnotes"""
    return list(res) if (res := split_annex(*add_ids(annex))) else None


def add_ids(annex: str) -> tuple[Optional[ResultSet], str]:
    """Initialize annex for splitting into individual footnotes"""
    html = make_soup(annex)
    if notes := html("sup"):
        for i, sup in enumerate(notes, start=1):
            sup["id"] = str(i)
    return notes or None, str(html)


def split_annex(
    notes: Optional[ResultSet], source: str
) -> Optional[Iterator[dict]]:
    """Assuming pre-formatted sup tags with ids, generate dictionaries of footnotes each consisting of a key, value"""
    if not notes:
        return None
    for note in notes:
        if text := note.get_text():
            yield {
                "key": get_digit_from_fn_pattern(text)
                or get_asterisk_from_fn_pattern(text),
                "value": make_value(snippetize(note, source)),
            }


def make_value(text: str) -> str:
    """Given an HTML string with the following format <sup> </sup> X X X,  remove the sup tag; clean the string"""

    html = make_soup(text)  # recreate soup from string slice
    if html.body.sup:  # use bs4 to remove sup tag from slice
        html.body.sup.decompose()  # remove the outer html

    result = html.body.extract()  # remove empty tags from result;
    # see https://stackoverflow.com/questions/33500888/how-to-remove-tags-that-have-no-content

    [x.decompose() for x in result.find_all(lambda tag: not tag.contents)]
    # breaking this down:
    # tag: not tag.contents = tags in the soup which are empty
    # this does not yet remove <p> </p> since this contains a whitespace
    # decompose each matching tag that fits the lambda function

    # the result is now clear of empty tags
    # convert result, a soup object, back to string and remove the body tags
    # body tags occur twice in the previous result
    cleaned = re.sub(r"\<(\/)?body\>", "", str(result), 2)
    return cleaned.strip()


def snippetize(note: Tag, source: str) -> str:
    """
    Slice a snippet from a soup object until the next sup tag, i.e. content between footnote id 7 to footnote id 8
    1. Each note tag has a unique ID;
    2. Re: start index, convert tag to a string; find this unique string in the source to get the index
    3. Re: end index, find the next sup tag, if none exists, then look for the body tag
    """
    s: int = source.index(str(note))
    mark: str = str(end) if (end := note.find_next("sup")) else "</body>"
    e: int = source.index(mark)
    return source[s:e]



from typing import Optional

from bs4.element import ResultSet
from lawsql_utils.html import (
    get_digit_from_fn_pattern,
    get_sup_tags,
    make_soup,
)

from .make import create_notes


def last_key_match(annex_notes: ResultSet, body_notes: ResultSet):
    return annex_notes[-1]["key"] == last_key(body_notes)


def last_key(tags: ResultSet) -> Optional[int]:
    """Get last integer of footnote, if it exists. Some cases have a '*' as ending footnote"""
    index = -1
    while True:
        try:  # traverse backwards, find first digit until end of list
            if digit := get_digit_from_fn_pattern(tags[index].get_text()):
                break
            index -= 1
        except IndexError:  # reach end of list without finding a digit
            return None  # deal 33674: all asterisks as footnotes
    return digit


def validated_footnotes(text: str, annex: str) -> Optional[list[dict]]:
    """
    Get validated list of footnotes from the annex
    1. Are there footnote numbers in the annex?
    2. Note that some cases do not contain footnotes
    3. Some improperly formatted footnotes are not yet covered
    4. Assuming footnote numbers, do they align? And have no issues?
    """
    if not (annex_notes := create_notes(annex)):
        return None

    if not (body_notes := get_sup_tags(text)):
        return None

    if not last_key_match(annex_notes, body_notes):
        return None

    if _error(annex_notes[-1]["value"]):
        annex_notes[-1]["value"] = "Error: could not format / retrieve."

    # Only return validated footnotes
    return annex_notes


def _error(text: str) -> bool:
    """
    Resolve edge case: is the last footnote value problematic?
    1. If it contains a sup tag, then yes
    2. If it contains a center tag, then possibly yes
    3. If it contains a strong tag, then possibly yes
    """
    html = make_soup(text)
    return (
        html("sup")
        or html("center")
        or html("strong")
        or "OPINION" in html.get_text()
    )
